Package: cancensus
Type: Package
Title: Access, Retrieve, and Work with Canadian Census Data and
        Geography
Version: 0.6.1
Authors@R: c(
    person("Jens", "von Bergmann", email = "jens@mountainmath.ca", role = c("aut"), comment = "API creator and maintainer"),
    person("Dmitry", "Shkolnik", email = "shkolnikd@gmail.com", role = c("aut", "cre"), comment = "Package maintainer, responsible for correspondence"),
    person("Aaron", "Jacobs", email = "atheriel@gmail.com", role = "aut"))
Description: Integrated, convenient, and uniform access to Canadian
    Census data and geography retrieved using the 'CensusMapper' API. This package produces analysis-ready 
    tidy data frames and spatial data in multiple formats, as well as convenience functions
    for working with Census variables, variable hierarchies, and region selection. API
    keys are freely available with free registration at <https://censusmapper.ca/api>.
    Census data and boundary geometries are reproduced and distributed on an "as
    is" basis with the permission of Statistics Canada (Statistics Canada 1996; 2001; 2006;
    2011; 2016; 2021).
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: yes
ByteCompile: yes
NeedsCompilation: no
Imports: digest (>= 0.1), dplyr (>= 1.0), httr (>= 1.0.0), jsonlite (>=
        1.0), sf, geojsonsf, rlang, readr
Suggests: knitr, ggplot2, leaflet, mapdeck, rmarkdown, scales, sp,
        tidyr, lwgeom, xml2, testthat (>= 3.0.0), microbenchmark
VignetteBuilder: knitr
URL: https://github.com/mountainMath/cancensus,
        https://mountainmath.github.io/cancensus/,
        https://censusmapper.ca/api
BugReports: https://github.com/mountainMath/cancensus/issues
Depends: R (>= 4.1)
Config/roxygen2/version: 8.0.0
Packaged: 2026-06-17 23:11:34 UTC; jens
Author: Jens von Bergmann [aut] (API creator and maintainer),
  Dmitry Shkolnik [aut, cre] (Package maintainer, responsible for
    correspondence),
  Aaron Jacobs [aut]
Maintainer: Dmitry Shkolnik <shkolnikd@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-19 06:50:08 UTC
Built: R 4.5.2; ; 2026-06-19 10:34:29 UTC; unix
