# Generated by roxygen2: do not edit by hand

export(add_unique_names_to_region_list)
export(as_census_region_list)
export(census_vectors)
export(child_census_vectors)
export(dataset_attribution)
export(explore_census_regions)
export(explore_census_vectors)
export(find_census_vectors)
export(get_census)
export(get_census_geometry)
export(get_intersecting_geometries)
export(get_statcan_geo_suite)
export(get_statcan_geographic_attributes)
export(get_statcan_geographies)
export(get_statcan_geography_relationships)
export(get_statcan_wds_data)
export(get_statcan_wds_metadata)
export(label_vectors)
export(list_cancensus_cache)
export(list_census_datasets)
export(list_census_regions)
export(list_census_vectors)
export(list_recalled_cached_data)
export(parent_census_vectors)
export(remove_from_cancensus_cache)
export(remove_recalled_cached_data)
export(search_census_regions)
export(search_census_vectors)
export(set_cancensus_api_key)
export(set_cancensus_cache_path)
export(show_cancensus_api_key)
export(show_cancensus_cache_path)
export(visualize_vector_hierarchy)
importFrom(dplyr,"%>%")
importFrom(rlang,.data)
importFrom(stats,setNames)
