#options("cancensus.base_url"="http://localhost:3000")
#options("cancensus.api_key"="CensusMapper_d79ad30f13e1daec24946aeaa8872d5d")
if (FALSE) {
  options("cancensus.base_url"="http://localhost:3000")
  Sys.setenv("CM_API_KEY"= 'CensusMapper_d79ad30f13e1daec24946aeaa8872d5d')
}
